@@ -16,7 +16,7 @@ print.markophylo <- function(x, ...) {
     if (x$results[[i]]$convergence == 0) {
       print.default(x$results[[i]]$parsep)
       cat("-----------------------------------\n")
-      if(x$reversible) {
+      if (x$reversible) {
         cat("Reversible option used. The root probabilities are estimated using maximum likelihood (see above for estimates and standard errors), and the rate matrix supplied to the function is symmetric (see above for estimates and standard errors). The estimated rate matrix can be written as the product of the symmetric matrix multiplied by a diagonal matrix (consisting of the estimated root probabilities). These rate estimates are:\n")
         mm_al2 <- x$results[[i]]$modelmat2
         rownames(mm_al2) <- colnames(mm_al2) <- x$alphabet
@@ -57,7 +57,7 @@ plottree <- function(x, colors = NULL, ...) {
   colo <- NULL
   for (i in 1:nrow(x$tree$edge)) {
     tmp <- which(unlist(lapply(lapply(x$bg, FUN = function(X) c(x$tree$edge[i, 1], x$tree$edge[i, 2]) %in% X), FUN = "all"), use.names = FALSE))
-    if((length(tmp) > 1)) {
+    if ((length(tmp) > 1)) {
       colo[i] <- tmp[which.min(unlist(lapply(x$bg, FUN = length)))]
     } else{
       colo[i] <- tmp
@@ -68,14 +68,14 @@ plottree <- function(x, colors = NULL, ...) {
 #########Check bg list of nodes argument###########
 checkbglist <- function(bg, tree){
   all_branches <- TRUE
-  for(i in 1:length(tree$edge.length)) {
-    if(!any(unlist(lapply(lapply(bg, FUN = function(j) tree$edge[i,] %in% j),FUN="all")))) all_branches <- FALSE
+  for (i in 1:length(tree$edge.length)) {
+    if (!any(unlist(lapply(lapply(bg, FUN = function(j) tree$edge[i,] %in% j),FUN = "all")))) all_branches <- FALSE
   }
   return(all_branches)
 }
 patterns <- function(x) { 
   #x is the matrix of phyletic patterns...
-  if(is.data.frame(x)) x <- as.matrix(x)
+  if (is.data.frame(x)) x <- as.matrix(x)
   wfn <- summary(as.factor(apply(x, 1, paste, collapse = " ")), maxsum = nrow(x))
   b <- attr(wfn, "names")
   ww <- unname(cbind(b, wfn))
@@ -90,22 +90,22 @@ patterns <- function(x) {
 }
 estimaterates <- function(usertree = NULL, userphyl = NULL, matchtipstodata = FALSE, unobserved = NULL, alphabet = NULL, modelmat = NULL, bgtype = "listofnodes", bg = NULL, partition = NULL, ratevar = FALSE, nocat = 4, reversible = FALSE, numhessian = TRUE,  rootprob = NULL, rpvec = NULL, lowli = 0.001, upli = 100, ...) {
   ptm <- proc.time()
-  if(ratevar!=FALSE) ratevar <- match.arg(ratevar,c("discgamma","partitionspecificgamma"))
+  if (ratevar != FALSE) ratevar <- match.arg(ratevar,c("discgamma","partitionspecificgamma"))
   bgtype <- match.arg(bgtype, c("listofnodes", "ancestornodes"))
-  if(!is.null(rootprob)) rootprob <- match.arg(rootprob, c("equal", "stationary", "maxlik", "user"))
-  if(is.character(modelmat)) modelmat <- match.arg(modelmat, c("ER","SYM","ARD","GTR","BD","BDER","BDSYM","BDARD"))
+  if (!is.null(rootprob)) rootprob <- match.arg(rootprob, c("equal", "stationary", "maxlik", "user"))
+  if (is.character(modelmat)) modelmat <- match.arg(modelmat, c("ER","SYM","ARD","GTR","BD","BDER","BDSYM","BDARD"))
   if (is.null(usertree)) 
     stop("usertree option is required.")
   if (is.null(userphyl) | !(is.matrix(userphyl) | is.data.frame(userphyl))) 
     stop("userphyl option is required.")
   if (!is.null(unobserved) & !is.matrix(userphyl)) 
     stop("unobserved should be a matrix.")
-  if(bgtype=="listofnodes" & !is.null(bg) & !is.list(bg))
+  if (bgtype == "listofnodes" & !is.null(bg) & !is.list(bg))
     stop("bg should be a vector with listofnodes argument.")
-  if(bgtype=="listofnodes" & !is.null(bg) & is.list(bg)){
-    if(!checkbglist(bg, usertree)) stop("Some branch is missing in bg.")
+  if (bgtype == "listofnodes" & !is.null(bg) & is.list(bg)) {
+    if (!checkbglist(bg, usertree)) stop("Some branch is missing in bg.")
   }
-  if(bgtype=="ancestornodes" & !is.null(bg) & !is.vector(bg))
+  if (bgtype == "ancestornodes" & !is.null(bg) & !is.vector(bg))
     stop("bg should be a list with ancestornodes argument.")
   if (is.null(alphabet)) 
     stop("alphabet option is required.")
@@ -117,119 +117,119 @@ estimaterates <- function(usertree = NULL, userphyl = NULL, matchtipstodata = FA
     stop("partition option must be a list.")
   if (ratevar == "partitionspecificgamma" & is.null(partition)) 
     stop("partition option must be specified and must be a list.")
-  if(!is.character(modelmat)) {
-    if(nrow(modelmat)!=ncol(modelmat))
+  if (!is.character(modelmat)) {
+    if (nrow(modelmat) != ncol(modelmat))
       stop("A square matrix should be provided for the modelmat option.")
-    if(nrow(modelmat)!=length(alphabet))
+    if (nrow(modelmat) != length(alphabet))
       stop("Length of alphabet supplied does not equal the number of states in the supplied rate matrix.")
   }
-  if(length(table(as.matrix(userphyl)))!=length(alphabet))
+  if (length(table(as.matrix(userphyl))) != length(alphabet))
     stop("Number of discrete states in the data provided does not equal the state space specified in the alphabet option.")
-  if(nrow(userphyl) < ncol(userphyl))
+  if (nrow(userphyl) < ncol(userphyl))
     warning("More taxa than patterns detected. Note that rows should represent the discrete character patterns and columns the different taxa. If this format was used, you can safely ignore this message.")
   #############Libraries###################
   libload <- function(funcval) {
     if (requireNamespace(paste(funcval), quietly = TRUE) == FALSE) 
-      stop("Please install package dependencies before continuing.")#install.packages(funcval, quiet = TRUE)
+      stop("Please install package dependencies before continuing.")
     suppressMessages(loadNamespace(funcval))
   }
   libload("ape")
   libload("numDeriv")
   libload("Rcpp")
   #########Build substitution rate index matrix#####################
-  buildindexmat <- function(type=modelmat,states=length(alphabet)){
-    if(type=="ER"){
+  buildindexmat <- function(type = modelmat, states = length(alphabet)) {
+    if (type == "ER") {
       #equal rates matrix
       modelm <- matrix(1,states,states)
       diag(modelm) <- NA
     }
-    if(type=="ARD"){
+    if (type == "ARD") {
       #all rates different matrix
       modelm <- matrix(0,states,states)
       diag(modelm) <- NA
-      modelm[!is.na(modelm)] <- 1:(states*(states-1))
+      modelm[!is.na(modelm)] <- 1:(states*(states - 1))
     }
-    if(type=="SYM"){
+    if (type == "SYM") {
       #symmetric matrix
       modelm <- matrix(0,states,states)
       diag(modelm) <- NA
-      modelm[lower.tri(modelm)] <- 1:(states*(states-1)/2)
+      modelm[lower.tri(modelm)] <- 1:(states*(states - 1)/2)
       modelm <- modelm + t(modelm)
     }
-    if(type=="GTR"){
+    if (type == "GTR") {
       #symmetric matrix
       modelm <- matrix(0,states,states)
       diag(modelm) <- NA
-      modelm[lower.tri(modelm)] <- 1:(states*(states-1)/2)
+      modelm[lower.tri(modelm)] <- 1:(states*(states - 1)/2)
       modelm <- modelm + t(modelm)
     }
-    if(type=="BD"){
+    if (type == "BD") {
       #birth death matrix
       modelm <- matrix(0,states,states)
       diag(modelm) <- NA
-      if(states > 2) {
+      if (states > 2) {
         diag(modelm[-1, ]) <- 1
         diag(modelm[, -1]) <- 2
       } 
-      if(states == 2){
+      if (states == 2) {
         modelm[2,1] <- 1
         modelm[1,2] <- 2
       }
     }
-    if(type=="BDER"){
+    if (type == "BDER") {
       #birth death equal rates matrix
       modelm <- matrix(0,states,states)
       diag(modelm) <- NA
-      if(states > 2) {
+      if (states > 2) {
         diag(modelm[-1, ]) <- 1
         diag(modelm[, -1]) <- 1
       }
-      if(states == 2) {
+      if (states == 2) {
         modelm[2,1] <- modelm[2,1] <- 1
       }
     }
-    if(type=="BDARD"){
+    if (type == "BDARD") {
       #birth death all rates different
       modelm <- matrix(0,states,states)
       diag(modelm) <- NA
-      if(states > 2){
-        diag(modelm[-1, ]) <- 1:(states-1)
-        diag(modelm[, -1]) <- (states):((states-1)*2)
+      if (states > 2) {
+        diag(modelm[-1, ]) <- 1:(states - 1)
+        diag(modelm[, -1]) <- (states):((states - 1)*2)
       }
-      if(states == 2){
-        modelm[2,1] <- 1
-        modelm[1,2] <- 2
+      if (states == 2) {
+        modelm[2, 1] <- 1
+        modelm[1, 2] <- 2
       }
     }
-    if(type=="BDSYM"){
+    if (type == "BDSYM") {
       #birth death symmetric
       modelm <- matrix(0,states,states)
       diag(modelm) <- NA
-      if(states > 2) {
-        diag(modelm[-1, ]) <- 1:(states-1)
+      if (states > 2) {
+        diag(modelm[-1, ]) <- 1:(states - 1)
         modelm <- modelm + t(modelm)
       } 
-      if(states == 2){
+      if (states == 2) {
         modelm[2,1] <- modelm[2,1] <- 1
       }
     }
     return(modelm)
   }
-  if(is.character(modelmat)) {
-    if(modelmat=="GTR") {
+  if (is.character(modelmat)) {
+    if (modelmat == "GTR") {
       reversible <- TRUE
       rootprob <- "maxlik"
     }
   }
-  if(is.character(modelmat)) modelmat <- buildindexmat()
+  if (is.character(modelmat)) modelmat <- buildindexmat()
   #########Transition rate matrix and substitution rate matrix#######
   TPM_taxa <- function(rates, ad, ti, rpin) {
     j <- unlist(lapply(lapply(bg, FUN = function(X) c(ad[1], ad[2]) %in% X), FUN = "all"), use.names = FALSE)
-    if(sum(j) > 1) j[j][-which.min(unlist(lapply(bg, FUN = length)))] <- FALSE
+    if (sum(j) > 1) j[j][-which.min(unlist(lapply(bg, FUN = length)))] <- FALSE
     for (gf in 1:length(rates[, j])) {
       Q[modelmat == gf] <- rates[gf, j]
     }
-    if(reversible & rootprob=="maxlik") Q <- sweep(Q, MARGIN = 2, FUN = "*", rpin, check.margin = FALSE)
+    if (reversible & rootprob == "maxlik") Q <- sweep(Q, MARGIN = 2, FUN = "*", rpin, check.margin = FALSE)
     diag(Q) <- -.rowSums(Q, m = al, n = al, na.rm = TRUE)
     return(expm(Q * ti))
   }
@@ -242,12 +242,12 @@ estimaterates <- function(usertree = NULL, userphyl = NULL, matchtipstodata = FA
   tree1 <- usertree
   tree1 <- ape::reorder.phylo(tree1, order = "postorder")
   
-  if(is.data.frame(userphyl)) userphyl <- as.matrix(userphyl)
+  if (is.data.frame(userphyl)) userphyl <- as.matrix(userphyl)
   diag(modelmat) <- NA
-  if(reversible & !isSymmetric(modelmat)) stop("modelmat should be symmetric to use with the reversible option.")
-  if(reversible & rootprob!="maxlik") stop("Root probabilities must be estimated using likelihood to use with the reversible option. Use maxlik option for rootprob.")
+  if (reversible & !isSymmetric(modelmat)) stop("modelmat should be symmetric to use with the reversible option.")
+  if (reversible & rootprob != "maxlik") stop("Root probabilities must be estimated using likelihood to use with the reversible option. Use maxlik option for rootprob.")
   
-  if(matchtipstodata) {
+  if (matchtipstodata) {
     fin <- userphyl[,pmatch(usertree$tip.label, colnames(userphyl))] 
     datab <- fin
   } else {
@@ -280,7 +280,6 @@ estimaterates <- function(usertree = NULL, userphyl = NULL, matchtipstodata = FA
     bg[[length(bg) + 1]] <- c(setdiff(1:(nooftaxa * 2 - 1), c(unlist(bg, use.names = FALSE))), bgo)
   }
   ##############Parameters and Variables################
-  ngenes <- phyl #number of different genes
   nointnodes <- nooftaxa - 1
   tips <- 1:nooftaxa
   len_tips <- length(tips)
@@ -458,21 +457,21 @@ estimaterates <- function(usertree = NULL, userphyl = NULL, matchtipstodata = FA
   Lixi <- matrix(0, nooftaxa + nointnodes, al)
   Lixi_init <- list()
   patlen <- unlist(lapply(X = w, FUN = length))
-  for(i in 1:length(w)){
+  for (i in 1:length(w)) {
     Lixi_init[[i]] <- rep(list(matrix(data = 0, nrow = nooftaxa + nointnodes, ncol = al)), length = patlen[i])
   } 
-  for(i in 1:length(w)){
-    for(j in 1:patlen[i]){
+  for (i in 1:length(w)) {
+    for (j in 1:patlen[i]) {
       for (u in alseq) {
         Lixi_init[[i]][[j]][which(databp_red[[i]][j, ] == alphabet[u]), u] <- 1
       }
     }
   }
-  if(!is.null(unobserved)){
+  if (!is.null(unobserved)) {
     Lixi_corr_init <- list()
     patlen_corr <- nrow(unobserved)
     Lixi_corr_init <- rep(list(matrix(data = 0, nrow = nooftaxa + nointnodes, ncol = al)), length = patlen_corr)
-      for(j in 1:patlen_corr){
+      for (j in 1:patlen_corr) {
         for (u in alseq) {
           Lixi_corr_init[[j]][which(unobserved[j, ] == alphabet[u]), u] <- 1
         }
@@ -483,7 +482,7 @@ estimaterates <- function(usertree = NULL, userphyl = NULL, matchtipstodata = FA
  
   ll <- function(model, pm_ll, rootp_ll, Lixi_in_ll) {
     tmp <- part_loopC(nocat, nodelist, al, tree1$edge[, 1], tree1$edge[, 2], pm_ll, Lixi_in_ll, len_tips)
-    logll <- log(rowSums(sweep(tmp, MARGIN=2, rootp_ll, `*`)))
+    logll <- log(rowSums(sweep(tmp, MARGIN = 2, rootp_ll, `*`)))
     return(logll)
   }
   pm <- list()
@@ -514,12 +513,12 @@ estimaterates <- function(usertree = NULL, userphyl = NULL, matchtipstodata = FA
     # pm is a list with the first component referring to the partitions, the second to the discrete gamma categories, and the third to the edge lengths.
 
     phy <- unlist_w * unlist(lapply(X = 1:psp, FUN = function(i) ll(model, pm[[i]], rootp = rootp[[i]], Lixi_in_ll = Lixi_init[[i]])), use.names = FALSE)
-    if (!is.null(unobserved) & is.null(partition)){
+    if (!is.null(unobserved) & is.null(partition)) {
       corr <- unlist(lapply(X = 1:psp, FUN = function(i) ll(model, pm[[i]], rootp = rootp[[i]], Lixi_in_ll = Lixi_corr_init)), use.names = FALSE)
       return(-(sum(phy) - nrow(databp) * log(1 - sum(exp(corr))) ) )
-    } else if (!is.null(unobserved) & !is.null(partition)){
+    } else if (!is.null(unobserved) & !is.null(partition)) {
       corr <- lapply(X = 1:psp, FUN = function(i) ll(model, pm[[i]], rootp = rootp[[i]], Lixi_in_ll = Lixi_corr_init))
-      return( -( sum(phy) - 
+      return( -(sum(phy) - 
                  sum(unlist(lapply(X = partition, FUN = length), 
                                   use.names = FALSE) * 
                  unlist(lapply(X = 
@@ -533,7 +532,6 @@ estimaterates <- function(usertree = NULL, userphyl = NULL, matchtipstodata = FA
     # When the likelihood is being conditioned on observable patterns only, and a partitioned analysis is being done, the multitude of lapplys are basically computing on lists the equivalent of log(1 - sum(exp(corr))) and then these are multiplied by a vector of number of observations in each partition.
     #-ve of the value because being minimized by default.
   }
-  vec <- numeric(al)
   rootpfun <- function(rootprob, rtype, currpar) {
     if (rootprob == "maxlik") 
       irootprob <- currpar$iroot
@@ -595,7 +593,6 @@ estimaterates <- function(usertree = NULL, userphyl = NULL, matchtipstodata = FA
   convcheck <- NULL
   modelnames <- "wop"
   modelmat2 <- NA
-  revcombined <- NA
   for (i in modelnames) {
     le_prev <- length(modelop[[i]]$start)
     res <- nlminb(start = modelop[[i]]$start, objective = totalll, model = i, lower = modelop[[i]]$lower, upper = modelop[[i]]$upper, 
@@ -610,20 +607,20 @@ estimaterates <- function(usertree = NULL, userphyl = NULL, matchtipstodata = FA
         res$hessian <- numDeriv::hessian(func = totalll, model = i, x = res$par)
     }
     res$parsep <- bamspforresult(res$par, i, partype = "par")
-    res$revcombined$rates <- array(data = NA, dim =c(sum(!is.na(modelmat) & modelmat!=0), csp, psp))
-    if(reversible) {
+    res$revcombined$rates <- array(data = NA, dim = c(sum(!is.na(modelmat) & modelmat != 0), csp, psp))
+    if (reversible) {
       Q_out <- modelmat
       modelmat2 <- modelmat
-      modelmat2[modelmat2!=0 & !is.na(modelmat2)] <- 1:(sum(!is.na(modelmat) & modelmat!=0))
-      for(pa in 1:psp){
-        for(ca in 1:csp){
+      modelmat2[modelmat2 != 0 & !is.na(modelmat2)] <- 1:(sum(!is.na(modelmat) & modelmat != 0))
+      for (pa in 1:psp) {
+        for (ca in 1:csp) {
           for (gf in 1:length(res$parsep$rates[,ca,pa])) {
             Q_out[modelmat == gf] <- res$parsep$rates[,ca,pa][gf]
           }
               Q_out <- sweep(Q_out, MARGIN = 2, FUN = "*", res$parsep$iroot, check.margin = FALSE) 
               #iroot is never a list because only ever used with "maxlik", never "stationary". 
               #the sweep method is ideal. However, because we are only indexing !is.na items, the multiplication by the diagonal matrix comprising the root probabilities method might also work...
-          res$revcombined$rates[,ca,pa] <- Q_out[which(!is.na(modelmat2) & modelmat2!=0)]
+          res$revcombined$rates[,ca,pa] <- Q_out[which(!is.na(modelmat2) & modelmat2 != 0)]
         }
       }
       res$modelmat2 <- modelmat2
@@ -639,7 +636,7 @@ estimaterates <- function(usertree = NULL, userphyl = NULL, matchtipstodata = FA
       }
       llhat <- -round(res$objective, 3)
       res$AIC <- 2 * llhat - 2 * res$df
-      res$BIC <- 2 * llhat - log(nooftaxa) * res$df
+      res$BIC <- 2 * llhat - log(phyl) * res$df
     }
     results[[i]] <- res
   }
@@ -653,7 +650,7 @@ estimaterates <- function(usertree = NULL, userphyl = NULL, matchtipstodata = FA
   }
   
   timetaken <- proc.time() - ptm
-  val <- list(call = match.call(), conv = convcheck, time = timetaken, bgtype = bgtype, bg = bg, results = results, tree = tree1, data_red = databp_red, alphabet = alphabet, reversible = reversible,  w = w, taxa = nooftaxa, phyl = ncol(databp), rootprob = rootprob, modelmat = modelmat, modelnames=modelnames)
+  val <- list(call = match.call(), conv = convcheck, time = timetaken, bgtype = bgtype, bg = bg, results = results, tree = tree1, data_red = databp_red, alphabet = alphabet, reversible = reversible,  w = w, taxa = nooftaxa, phyl = phyl, rootprob = rootprob, modelmat = modelmat, modelnames = modelnames)
   class(val) <- "markophylo"
   return(invisible(val))
 }
\ No newline at end of file
